/***
 * Copyright (C) 1996-98. by Miroslav Zagorac <zaga@tirnanog.cc.fer.hr>
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <dirent.h>
#include <unistd.h>
#include <errno.h>
#include <sysexits.h>
#include <sys/types.h>
#include <sys/param.h>

#include "../lib/lib_debug.h"
#include "../lib/lib_image.h"

#include "make_objects_lib.h"


FILE  *fd_C = NULL, *fd_H = NULL;
DIR   *dir = NULL;
short  program_error = 1;


#ifdef ANSI_FUNC

void read_palette (char *filename)
#else

void read_palette (filename)
   char *filename;
#endif
{
   palette_t *palette;

   if ((palette = image_palette_load (filename)))
   {
      short i;

      (void) fprintf (fd_H, "extern unchar bd_palette[];\n");
      (void) fprintf (fd_C, "\n\n\nunchar bd_palette[] =\n{");

      for (i = 0; i < SIZEOF_PALETTE; i++)
         (void) fprintf (fd_C, "%s0x%02x, ", (i % 3) ? "" : "\n   ", palette[i]);

      (void) fprintf (fd_C, "\n};");

      (void) printf ("P");
      (void) fflush (stdout);
   }
}


#ifdef ANSI_FUNC

void read_dir (char *directory, char *prefix)
#else

void read_dir (directory, prefix)
   char *directory;
   char *prefix;
#endif
{
   struct dirent *dir_entry;

   if ((dir = opendir (directory)))
   {
      while ((dir_entry = readdir (dir)))
      {
         struct IMAGE_DATA *image;
         char    filename[MAXPATHLEN];
         long    i;

         (void) sprintf (filename, "%s/%s", directory, dir_entry->d_name);

         if ((image = image_load (filename, NULL, 0)))
         {
            (void) fprintf (fd_H, "extern unchar %s%.*s[];\n", prefix, (int) strlen (dir_entry->d_name) - 4, dir_entry->d_name);
            (void) fprintf (fd_C, "\n\n\nunchar %s%.*s[] =\n{", prefix, (int) strlen (dir_entry->d_name) - 4, dir_entry->d_name);

            for (i = 0; i < (image->width * image->height); i++)
               (void) fprintf (fd_C, "%s0x%02x, ", (i % 32) ? "" : "\n   ", image->image[i]);

            (void) fprintf (fd_C, "\n};");

            (void) printf (".");
            (void) fflush (stdout);

            image_free (&image);
         }
      }

      (void) closedir (dir);

      dir = NULL;
   }
   else
      perror (directory);
}


#ifdef ANSI_FUNC

static void my_exit (void)
#else
   
static void my_exit ()
#endif
{
   if (fd_C)
   {
      (void) fclose (fd_C);

      fd_C = NULL;
   }

   if (fd_C)
   {
      (void) fclose (fd_H);

      fd_H = NULL;
   }

   if (dir)
   {
      (void) closedir (dir);

      dir = NULL;
   }

   if (program_error)
   {
      (void) unlink (OBJECTS_DEF_H);
      (void) unlink (OBJECTS_DEF_C);
   }
}


#ifdef ANSI_FUNC

int main (int argc, char **argv)
#else

int main (argc, argv)
   int argc;
   char **argv;
#endif
{
   debug_off ();

   (void) atexit (my_exit);

   (void) printf ("Generating objects library: ");
   (void) fflush (stdout);

   if (! (fd_H = fopen (OBJECTS_DEF_H, "w+t")))
      perror_exit (OBJECTS_DEF_H, errno);

   if (! (fd_C = fopen (OBJECTS_DEF_C, "w+t")))
      perror_exit (OBJECTS_DEF_C, errno);

   (void) fprintf (fd_H, "/***\n * Generated automatically by %s.\n", *argv);
   (void) fprintf (fd_H, " * Do not edit!\n */\n#ifndef _OBJECTS_DEF_H_\n#define _OBJECTS_DEF_H_\n\n");
   (void) fprintf (fd_H, "#ifdef HAVE_CONFIG_H\n#  include <config.h>\n#endif\n\n");
   (void) fprintf (fd_H, "#include <sys/types.h>\n\n\n");

   (void) fprintf (fd_C, "/***\n * Generated automatically by make_objects.\n");
   (void) fprintf (fd_C, " * Do not edit!\n */\n#include \"%s\"", OBJECTS_DEF_H);

   read_palette ("palettes/BoulderDash");
   read_dir ("spectrum", "spectrum_");
   read_dir ("commodore", "commodore_");
   read_dir ("default", "default_");
   read_dir ("graphics", "");

   (void) fprintf (fd_H, "\n#endif\n");
   (void) fprintf (fd_C, "\n");

   (void) printf (" DONE!\n");

   program_error = 0;

   return (0);
}
